@@ -35,6 +35,12 @@ extern "C" {
 #include <mrn_operations.hpp>
 #include <mrn_database.hpp>
 
+#if __cplusplus >= 201402
+#  define mrn_override override
+#else
+#  define mrn_override
+#endif
+
 #if (MYSQL_VERSION_ID >= 50514 && MYSQL_VERSION_ID < 50600)
 #  define MRN_HANDLER_HAVE_FINAL_ADD_INDEX 1
 #endif
@@ -104,6 +110,13 @@ extern "C" {
 #  define MRN_HAVE_HA_EXTRA_PREPARE_FOR_FORCED_CLOSE
 #endif
 
+#if (!defined(MRN_MARIADB_P) && MYSQL_VERSION_ID >= 80002)
+#define MRN_HAVE_HA_EXTRA_SKIP_SERIALIZABLE_DD_VIEW
+#define MRN_HAVE_HA_EXTRA_BEGIN_ALTER_COPY
+#define MRN_HAVE_HA_EXTRA_END_ALTER_COPY
+#define MRN_HAVE_HA_EXTRA_NO_AUTOINC_LOCKING
+#endif
+
 #if MYSQL_VERSION_ID >= 50607 && \
     (!defined(MRN_MARIADB_P) || MYSQL_VERSION_ID < 100008)
 #  define MRN_HAVE_HA_EXTRA_EXPORT
@@ -227,6 +240,28 @@ extern "C" {
 #  define MRN_HANDLER_RECORDS_RETURN_ERROR
 #endif
 
+#if MYSQL_VERSION_ID < 80002 || defined(MRN_MARIADB_P)
+#  define MRN_HANDLER_HAVE_KEYS_TO_USE_FOR_SCANNING
+#endif
+
+#if (!defined(MRN_MARIADB_P) && MYSQL_VERSION_ID >= 80002)
+#  define MRN_ST_MYSQL_PLUGIN_HAVE_CHECK_UNINSTALL
+#endif
+
+#if (!defined(MRN_MARIADB_P) && MYSQL_VERSION_ID >= 80002)
+#  define MRN_HANDLER_OPEN_HAVE_TABLE_DEFINITION
+#  define MRN_HANDLER_CREATE_HAVE_TABLE_DEFINITION
+#endif
+
+#if (!defined(MRN_MARIADB_P) && MYSQL_VERSION_ID >= 80002)
+#  define MRN_HANDLERTON_CREATE_HAVE_PARTITIONED
+#endif
+
+#if defined(HAVE_PSI_INTERFACE) &&                      \
+  (MYSQL_VERSION_ID < 80002 || defined(MRN_MARIADB_P))
+#  define MRN_HAVE_PSI_SERVER
+#endif
+
 class ha_mroonga;
 
 /* structs */
@@ -382,8 +417,20 @@ public:
   ulonglong table_flags() const;                                   // required
   ulong index_flags(uint idx, uint part, bool all_parts) const;    // required
 
-  int create(const char *name, TABLE *form, HA_CREATE_INFO *info); // required
-  int open(const char *name, int mode, uint open_options);         // required
+  // required
+  int create(const char *name, TABLE *form, HA_CREATE_INFO *info
+#ifdef MRN_HANDLER_CREATE_HAVE_TABLE_DEFINITION
+             ,
+             dd::Table *table_def
+#endif
+    ) mrn_override;
+  // required
+  int open(const char *name, int mode, uint open_options
+#ifdef MRN_HANDLER_OPEN_HAVE_TABLE_DEFINITION
+           ,
+           const dd::Table *table_def
+#endif
+    ) mrn_override;
 #ifndef MRN_HANDLER_HAVE_HA_CLOSE
   int close();                                                     // required
 #endif
@@ -489,7 +536,9 @@ public:
   int truncate();
   double scan_time();
   double read_time(uint index, uint ranges, ha_rows rows);
+#ifdef MRN_HANDLER_HAVE_KEYS_TO_USE_FOR_SCANNING
   const key_map *keys_to_use_for_scanning();
+#endif
   ha_rows estimate_rows_upper_bound();
   void update_create_info(HA_CREATE_INFO* create_info);
   int rename_table(const char *from, const char *to);
@@ -623,6 +672,9 @@ private:
   int drop_indexes_multiple(const char *table_name, grn_obj *table,
                             const char *index_table_name_separator);
   int drop_indexes(const char *table_name);
+  bool find_table_flags(HA_CREATE_INFO *info,
+                        MRN_SHARE *mrn_share,
+                        grn_table_flags *table_flags);
   bool find_column_flags(Field *field, MRN_SHARE *mrn_share, int i,
                          grn_obj_flags *column_flags);
   grn_obj *find_column_type(Field *field, MRN_SHARE *mrn_share, int i,
@@ -802,7 +854,10 @@ private:
                                grn_obj **index_tables,
                                grn_obj **index_columns,
                                MRN_SHARE *tmp_share);
-  int wrapper_create_index(const char *name, TABLE *table, MRN_SHARE *tmp_share);
+  int wrapper_create_index(const char *name,
+                           TABLE *table,
+                           HA_CREATE_INFO *info,
+                           MRN_SHARE *tmp_share);
   int storage_create_validate_pseudo_column(TABLE *table);
 #ifdef MRN_SUPPORT_FOREIGN_KEYS
   bool storage_create_foreign_key(TABLE *table, const char *grn_table_name,
@@ -972,50 +1027,18 @@ private:
   FT_INFO *storage_ft_init_ext(uint flags, uint key_nr, String *key);
   void generic_ft_init_ext_add_conditions_fast_order_limit(
       struct st_mrn_ft_info *info, grn_obj *expression);
-  bool generic_ft_init_ext_parse_pragma_d(struct st_mrn_ft_info *info,
-                                          const char *keyword,
-                                          uint keyword_length,
-                                          grn_operator *default_operator,
-                                          uint *consumed_keyword_length);
-  void generic_ft_init_ext_parse_pragma_w_append_section(
-    struct st_mrn_ft_info *info,
-    grn_obj *index_column,
-    grn_obj *match_columns,
-    uint section,
-    grn_obj *section_value_buffer,
-    int weight,
-    uint n_weights);
-  bool generic_ft_init_ext_parse_pragma_w(struct st_mrn_ft_info *info,
-                                          const char *keyword,
-                                          uint keyword_length,
-                                          grn_obj *index_column,
-                                          grn_obj *match_columns,
-                                          uint *consumed_keyword_length,
-                                          grn_obj *tmp_objects);
-  grn_expr_flags expr_flags_in_boolean_mode();
-  void generic_ft_init_ext_parse_pragma(struct st_mrn_ft_info *info,
-                                        String *key,
-                                        grn_obj *index_column,
-                                        grn_obj *match_columns,
-                                        const char **keyword,
-                                        uint *keyword_length,
-                                        grn_operator *default_operator,
-                                        grn_expr_flags *flags,
-                                        grn_obj *tmp_objects);
   grn_rc generic_ft_init_ext_prepare_expression_in_boolean_mode(
     struct st_mrn_ft_info *info,
     String *key,
     grn_obj *index_column,
     grn_obj *match_columns,
-    grn_obj *expression,
-    grn_obj *tmp_objects);
+    grn_obj *expression);
   grn_rc generic_ft_init_ext_prepare_expression_in_normal_mode(
     struct st_mrn_ft_info *info,
     String *key,
     grn_obj *index_column,
     grn_obj *match_columns,
-    grn_obj *expression,
-    grn_obj *tmp_objects);
+    grn_obj *expression);
   struct st_mrn_ft_info *generic_ft_init_ext_select(uint flags,
                                                     uint key_nr,
                                                     String *key);
@@ -1093,8 +1116,10 @@ private:
   double storage_scan_time();
   double wrapper_read_time(uint index, uint ranges, ha_rows rows);
   double storage_read_time(uint index, uint ranges, ha_rows rows);
+#ifdef MRN_HANDLER_HAVE_KEYS_TO_USE_FOR_SCANNING
   const key_map *wrapper_keys_to_use_for_scanning();
   const key_map *storage_keys_to_use_for_scanning();
+#endif
   ha_rows wrapper_estimate_rows_upper_bound();
   ha_rows storage_estimate_rows_upper_bound();
   void wrapper_update_create_info(HA_CREATE_INFO* create_info);
